package com.ningdatech.pmapi.irs.manage;

import cn.hutool.core.collection.CollUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.ningdatech.basic.function.VUtils;
import com.ningdatech.pmapi.common.enumeration.CommonEnum;
import com.ningdatech.pmapi.common.statemachine.contants.RegionContant;
import com.ningdatech.pmapi.common.statemachine.util.StateMachineUtils;
import com.ningdatech.pmapi.common.util.CryptUtils;
import com.ningdatech.pmapi.common.util.RefreshKeyUtil;
import com.ningdatech.pmapi.irs.model.dto.*;
import com.ningdatech.pmapi.irs.model.res.ApiResponse;
import com.ningdatech.pmapi.projectlib.model.entity.Project;
import com.ningdatech.pmapi.projectlib.model.entity.ProjectApplication;
import com.ningdatech.pmapi.projectlib.service.IProjectApplicationService;
import com.ningdatech.pmapi.projectlib.service.IProjectService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Component;
import org.springframework.web.client.RestTemplate;

import java.security.NoSuchAlgorithmException;
import java.time.LocalDateTime;
import java.util.Collections;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @Classname AppIrsManage
 * @Description
 * @Date 2023/7/12 11:43
 * @Author PoffyZhang
 */
@Component
@Slf4j
@RequiredArgsConstructor
public class AppIrsManage {

    private final IProjectService projectService;

    private final IProjectApplicationService applicationService;

    private final StateMachineUtils stateMachineUtils;

    @Value("${irs.push-app.appKey}")
    private String pushAppKey;

    @Value("${irs.push-app.appScret}")
    private String pushAppScret;

    @Value("${irs.push-app.url}")
    private String pushUrl;

    @Value("${irs.search-app.appKey}")
    private String searchAppKey;

    @Value("${irs.search-app.appScret}")
    private String searchAppScret;

    @Value("${irs.search-app.url}")
    private String searchUrl;

    /**
     * 推送 项目和应用关系
     * @param dto
     * @return
     * @throws NoSuchAlgorithmException
     */
    public Boolean pushProjectApp(PushProjectAppToIrsDTO dto) throws NoSuchAlgorithmException {
        String appCode = dto.getAppCode();
        Long appId = dto.getAppId();
        String projectCode = dto.getProjectCode();
        ProjectApplication app = applicationService.getById(appId);
        VUtils.isTrue(Objects.isNull(app)).throwMessage("应用不存在!");
        VUtils.isTrue(CommonEnum.NO.getCode().equals(app.getIsFirst())).throwMessage("应用不是初次新建!");
        Project project = projectService.getProjectByCode(projectCode);
        VUtils.isTrue(Objects.isNull(project)).throwMessage("项目不存在!");

        ApiApplyDTO apiApply = new ApiApplyDTO();
        apiApply.setAppId(appCode);
        JSONObject jsonObject = this.searchApp(apiApply);
        ApiApplySearchResult apiApplySearchResult = JSON.parseObject(JSON.toJSONString(jsonObject),ApiApplySearchResult.class);
        VUtils.isTrue(Objects.isNull(apiApplySearchResult)).throwMessage("应用在IRS不存在!");

        long timestamp = System.currentTimeMillis();
        String areaCode = project.getAreaCode();
        String projectName = project.getProjectName();
        String requestSecret = RefreshKeyUtil.getRequestSecret(pushAppKey, pushAppScret);
        String capCode = CryptUtils.MD5Encode(timestamp + areaCode);
        String capTime = String.valueOf(timestamp);
        String sign = CryptUtils.MD5Encode(pushAppKey + requestSecret + timestamp);
        String url = pushUrl + "?requestTime=" + timestamp +
                "&sign=" + sign + "&appKey=" + pushAppKey + "&capCode=" + capCode + "&capTime=" + capTime +
                "&baseProjSys=" + apiApplySearchResult.getName() + "&areaCode=" + areaCode + "&baseProjName=" + projectName +
                "&baseProjId=" + project.getProjectCode() +
                "&isEffective=1&baseProjSysCode=" + appCode;
        RestTemplate restTemplate = new RestTemplate();
        ResponseEntity<ApiResponse> forEntity = restTemplate.getForEntity(url, ApiResponse.class);

        log.info("推送 项目和应用结果 : {}", JSON.toJSONString(forEntity));

        ApiResponse body = forEntity.getBody();

        if(Objects.isNull(body) || Objects.isNull(body.getCode())
                || 0 != body.getCode()){
            return Boolean.FALSE;
        }

        //保存关系
        app.setAppCode(appCode);
        if(applicationService.updateById(app)){
            /**
             * 判断下 如果成功的话 那么判断下 这个项目的初次创建的app 是否都已经被注册
             * 如果都已经注册 要调用状态机 进入下一个状态
             */
            List<ProjectApplication> apps = applicationService.list(Wrappers.lambdaQuery(ProjectApplication.class)
                    .eq(ProjectApplication::getProjectId, project.getId()));
            //筛选出 初次新建的app 但是没有appCode的
            apps = apps.stream().filter(a -> {
                        if(CommonEnum.YES.getCode().equals(a.getIsFirst()) &&
                                StringUtils.isBlank(app.getAppCode())){
                            return Boolean.TRUE;
                        }
                        return Boolean.FALSE;
                    })
                    .collect(Collectors.toList());

            //如果筛选出来的 为空的话 就调用状态机
            if(CollUtil.isEmpty(apps)){
                stateMachineUtils.pass(project);
                project.setUpdateOn(LocalDateTime.now());
                projectService.updateById(project);
            }
        }

        return body.getSuccess();
    }

    /**
     * 查询app
     * @param apply
     * @return
     * @throws NoSuchAlgorithmException
     */
    public JSONObject searchApp(ApiApplyDTO apply) throws NoSuchAlgorithmException {
        long timestamp = System.currentTimeMillis();
        String appCode = apply.getAppId();
        String requestSecret = RefreshKeyUtil.getRequestSecret(searchAppKey, searchAppScret);
        String sign = CryptUtils.MD5Encode(searchAppKey + requestSecret + timestamp);
        String url = searchUrl + "?requestTime=" + timestamp + "&sign=" + sign +
                "&appKey=" + searchAppKey + "&" +
                "pageSize=10&pageNum=1&appCode=" + appCode +
                "&areaCode=&deptCode=&name=";
        log.info("url : {}",url);
        RestTemplate restTemplate = new RestTemplate();
        ResponseEntity<String> forEntity = restTemplate.getForEntity(url, String.class);

        log.info("查询应用目录 : {}",JSON.toJSONString(forEntity));

        if(Objects.nonNull(forEntity.getBody()) && Objects.nonNull(forEntity.getBody())){
            JSONObject body = JSON.parseObject(forEntity.getBody());
            if(Objects.isNull(body)){
                return null;
            }
            JSONObject datas = body.getJSONObject("datas");
            if(Objects.isNull(datas)){
                return null;
            }
            JSONObject data = datas.getJSONObject("data");
            if(Objects.isNull(data)){
                return null;
            }
            JSONArray apiApplySearchResult = data.getJSONArray("apiApplySearchResult");
            if(CollUtil.isEmpty(apiApplySearchResult)){
                return null;
            }
            return apiApplySearchResult.getJSONObject(0);
        }
        return null;
    }

    public JSONArray searchApps(ApiApplyDTO apply) throws NoSuchAlgorithmException {
        long timestamp = System.currentTimeMillis();
        String areaCode = StringUtils.isNotBlank(apply.getAreaCode()) ?
                apply.getAreaCode() : null;
        String requestSecret = RefreshKeyUtil.getRequestSecret(searchAppKey, searchAppScret);
        String sign = CryptUtils.MD5Encode(searchAppKey + requestSecret + timestamp);
        String url = searchUrl + "?requestTime=" + timestamp + "&sign=" + sign +
                "&appKey=" + searchAppKey + "&pageSize=1000&" +
                "pageNum=1&areaCode=" + areaCode +
                "&deptCode=&name=";
        RestTemplate restTemplate = new RestTemplate();
        ResponseEntity<String> forEntity = restTemplate.getForEntity(url, String.class);

        log.info("查询应用目录 : {}",JSON.toJSONString(forEntity));

        if(Objects.nonNull(forEntity.getBody())){
            JSONObject body = JSON.parseObject(forEntity.getBody());
            if(Objects.isNull(body)){
                return null;
            }
            JSONObject datas = body.getJSONObject("datas");
            if(Objects.isNull(datas)){
                return null;
            }
            JSONObject data = datas.getJSONObject("data");
            if(Objects.isNull(data)){
                return null;
            }
            JSONArray apiApplySearchResult = data.getJSONArray("apiApplySearchResult");
            return apiApplySearchResult;
        }
        return null;
    }
}
